﻿//--------------------------------------------------------------
//	シリアルポートと関連して使うクラス
//
//		作成者：三上直樹，2021/08/01 作成，©三上直樹 2021
//--------------------------------------------------------------

using System;
using System.Windows.Forms;
using System.IO.Ports;

namespace Mikami.ComPort
{
	// シリアルポートに関連するクラス
	public class MySerialPort
	{
		public SerialPort SrPort { get; private set; }	// シリアルポート

		// コンストラクタ
		public MySerialPort(SerialPort port) { SrPort = port; }

		// シリアルポートのオープン処理
		public bool Open(string name)
		{
			SrPort.PortName = ComPort.GetNumber(name);
			try {
				SrPort.Open();
				return true;
			}
			catch {
				System.Threading.Thread.Sleep(50);
				return false;
			}
		}

		// シリアルポートに文字列を送る
		public void SendTo(string str)
		{
			try { SrPort.Write(str + "\r"); }
			catch (Exception ex) { ExceptionCatch(ex); }
		}

		// Debug 時のみ例外のメッセージを出すようにするために使用
		public void ExceptionCatch(Exception ex)
		{
			#if DEBUG
			MessageBox.Show(ex.Message + "," + ex.Source + "," +
							ex.TargetSite.ToString());
			#endif
		}
	}
}
